from transformers import AutoModelForSequenceClassification

MODEL = {"fine_tune_base_model": AutoModelForSequenceClassification}


class Model:
    def __init__(self):
        super().__init__()

    @classmethod
    def from_config(cls, mode: str):
        try:
            class_ = MODEL[mode]
        except KeyError:
            raise KeyError(f'Mode type "{mode}" is not implemented.')

        return class_



from typing import Dict

import pandas as pd
from tqdm import tqdm


def get_label_counts(df: pd.DataFrame) -> Dict[str, int]:
    label_count = {}
    for i in df:
        k, v = list(i[0].value_counts().to_dict().items())[0]
        if k not in label_count:
            label_count[k] = 0
        label_count[k] += v
    return label_count


def get_dataset_length(file_name: str, split_type: str) -> int:
    data_files = get_file_names(file_name)
    file_map = {"train": data_files[0], "valid": data_files[1], "test": data_files[2]}
    return sum([len(i) for i in pd.read_csv(file_map[split_type], encoding="latin-1", chunksize=1000)])


def get_file_names(file_name: str) -> tuple[str, str, str]:
    file_base_pth, file_ext = file_name.rsplit(".", 1)
    train_file = f"{file_base_pth}.train.{file_ext}"
    valid_file = f"{file_base_pth}.valid.{file_ext}"
    test_file = f"{file_base_pth}.test.{file_ext}"
    return train_file, valid_file, test_file


def split_dataset(file: str, train_ratio: float = 0.7, test_ratio: float = 0.15, valid_ratio: float = 0.15) -> bool:
    train_file, valid_file, test_file = get_file_names(file)
    df = pd.read_csv(file, encoding="latin-1", header=None, chunksize=1000)
    # label_count = get_label_counts(df)
    total_chunks = sum([len(i) for i in df])
    pbar = tqdm(total=total_chunks, desc="Processing chunks", unit="chunk")
    for chunk in pd.read_csv(file, encoding="latin-1", header=None, chunksize=1000):
        train_idx = int(train_ratio * len(chunk))
        valid_idx = int(valid_ratio * len(chunk))
        test_idx = int(test_ratio * len(chunk))
        chunk.iloc[:train_idx, :].to_csv(train_file, mode="a", header=False, index=False)
        chunk.iloc[train_idx : train_idx + valid_idx, :].to_csv(valid_file, mode="a", header=False, index=False)
        chunk.iloc[train_idx + valid_idx : train_idx + valid_idx + test_idx, :].to_csv(
            test_file, mode="a", header=False, index=False
        )
        pbar.update(1000)
    return True


if __name__ == "__main__":
    raw_dataset_file = (
        "/home/ppradhan/Documents/my_learnings/my_uni_stuffs/sa_data_storage/training.1600000"
        ".processed.noemoticon.csv"
    )
    print(split_dataset(raw_dataset_file))



from typing import Dict

import torch
import yaml
from sa_app.common.utils import load_mapping, parse_args
from sa_app.data.data import InitializeDataset
from sa_app.data.data_cleaner import StackedPreprocessor
from transformers import AutoModelForSequenceClassification, AutoTokenizer


class InferenceEngine:
    def __init__(self, inference_params: Dict, training_params: Dict, dataset_params: Dict, device: str):
        # TODO : Download artifacts from wandb
        # Set the device to CPU or GPU
        self.device = device

        # Load the trained model and tokenizer
        self.model_path = inference_params["model_dir"]
        self.model = AutoModelForSequenceClassification.from_pretrained(self.model_path).to(self.device)
        self.tokenizer = AutoTokenizer.from_pretrained(training_params["tokenizer"])
        self.preprocessors = StackedPreprocessor(dataset_params["preprocessors"])
        dataset_obj = InitializeDataset(dataset_params)
        _, label_mapping_path = dataset_obj()
        self.label_mapping = load_mapping(label_mapping_path)

    def perform_inference(self, sentence):
        # Preprocess the input sentence
        sentence = self.preprocessors([sentence])[0]

        # Tokenize the input sentence
        inputs = self.tokenizer(sentence, truncation=True, padding=True, return_tensors="pt")
        inputs = inputs.to(self.device)

        # Forward pass through the model
        outputs = self.model(**inputs)

        # Get the predicted labels
        predicted_labels = torch.argmax(outputs.logits, dim=1)

        return self.label_mapping[predicted_labels.tolist()[0]]


if __name__ == "__main__":
    args = parse_args()
    config = yaml.safe_load(open(args.config, "r"))
    device_in_use = "cuda" if torch.cuda.is_available() else "cpu"
    # Example usage
    ie_obj = InferenceEngine(
        inference_params=config["inference_params"],
        training_params=config["training_params"],
        dataset_params=config["dataset_params"],
        device=device_in_use,
    )
    input_sentence = "I feel so bad today . Such a bad day :( "
    predicted_labels = ie_obj.perform_inference(input_sentence)
    print("Predicted labels:", predicted_labels)



import argparse
import csv
from typing import Dict, List

from pytorch_lightning.loggers import WandbLogger


def init_model_loggers(dataset_params: Dict, training_params: Dict) -> List[WandbLogger]:
    wandb_project_name = dataset_params.get("wandb_storage").get("wandb_project_name")
    log_dir = training_params.get("logging").get("log_dir")
    wandb_logger = WandbLogger(project=wandb_project_name, save_dir=log_dir)
    return [
        wandb_logger,
    ]


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--config",
        default="sa_app/app_cfg.yml",
        type=str,
        help="Path to config",
    )
    parser.add_argument(
        "--mode",
        default="train",
        type=str,
        help="mode of execution",
    )

    return parser.parse_args()


def load_mapping(label_files_path) -> List[str]:
    with open(label_files_path) as labels_file:
        html = labels_file.read().split("\n")
        csvreader = csv.reader(html, delimiter="\t")
    return [row[1] for row in csvreader if len(row) > 1]
